unit photodrv;
{ this unit replaces Computer Boards' TP_PDMA.OBJ file of
	interface routines for the Metrabyte pdma-16 board
	with code more specific to the Pico Systems
	Photoplotter  --  it is to operate the PDMA card under
	Windows 3.x }

interface
uses photogbl,WinCrt,WinDos,WinProcs,WinTypes,win31;
{$I photodef.pas}

  procedure write_control(value : integer);
  procedure wait_count;
  Function Plotter_status: integer;
  procedure step_head(direction : dir_type);
  function reference_head : boolean;
  procedure init;
  procedure define_bands;
  procedure send_band ( which_band : integer);
  procedure check_band(which_band : integer);
  procedure return_mem;
  procedure stop_motors;

implementation

const
 { register addresses }
  pdma_a	= $300;
  pdma_b	= pdma_a + 1;
  pdma_dma	= pdma_a + 2;
  pdma_int	= pdma_a + 3;
  pdma_cnt0	= pdma_a + 4;
  pdma_cnt1	= pdma_a + 5;
  pdma_cnt2	= pdma_a + 6;
  pdma_cntcsr	= pdma_a + 7;

{ dma reg bit defs }
dma_adir	= 1;
dma_bdir	= 2;
dma_word	= 4;
dma_timer	= 8;
dma_aux1	=16;
dma_aux2	=32;
dma_lev3	=64;
dma_enab	=128;

{ interrupt control reg bit defs }
int_minus   =  1;    { 1 = - edge }
int_src0    =  2;	{ 00 = ext,   01 = 8237 TC }
int_src1    =  4;	{ 10 = timer, 11 = 8237 TC }
int_aux3    =  8;
int_lev0    = 16;	{ 3 bit int level sel }
int_lev1    = 32;
int_lev2    = 64;
int_enab    =128;

{ counter ctrl reg definitions }
ctr_bcd		=  1;
ctr_m0		=  2;	{ mode  0  pulse on TC 		3  square wave gen }
ctr_m1		=  4;	{ mode  1  prog 1 shot		4  software trig strobe }
ctr_m2		=  8;	{ mode  2  rate generator		5  hardware trig strobe }
ctr_rl0		= 16;	{read/load ctrl 0 = counter latch	2 = do LSB }
ctr_rl1		= 32;	{	         1 = do MSB	3 = do LSB, then MSB }
ctr_sc0		= 64;	{ ctr sel  0 = counter 0		2 = counter 2 }
ctr_sc1		=128;	{            1 = counter 1		3 = readback command }

  int_level = 7;		{ interrupt level }
  DMA_level = 1;               	{ DMA channel to use }

  { define some registers in system's DMA controller }
dc_br1	=	2;	{ level 1 base reg }
dc_bc1	=	3;	{ level 1 byte count reg }
dc_br3	=	6;	{ level 3 base reg }
dc_bc3	=	7;	{ level 3 byte count reg }
dc_mask =	$0A;	{ 8237 mask reg }
dc_mode =	$0B;	{ 8237 mode reg }
dc_bptr	=	$0C;	{ 8237 byte pointer reg }
dc_page1 =	$83;	{ DMA ch 1 page reg }
dc_page3 =	$82;	{ DMA ch 3 page reg }

  { define registers in system's 8259 Interrupt controller }

int_mask   =    $021;   { 8259 interrupt controller mask register }

type
  descrip_type = array [0..1] of longint; { a descriptor table entry }

  dds_rec = record            { DMA Descriptor Data Structure }
    Region_Size : longint;
    Offset      : longint;
    Selector    : word;
    Buffer_ID   : word;
    Phys_addr   : longint;
  end;

  VDS_Block_rec = record        { block for VDS get version result }
    maj_min,prod,rev,dma_lo,dma_hi,flags,error : integer;
  end;

{procedure return_mem ; forward;}

{ *section - routines to access photoplotter hardware follow }
procedure write_control (value : integer) ;
begin
  writeln(output,'write_control sends ',value,' to a reg.');
  port[pdma_a] := value;
  i := port[pdma_dma];
  writeln(output,'write_control reads ',i,' from dma reg.');
  writeln(output,'write_control sends ',i and not dma_aux1,' to dma reg.');
  port[pdma_dma] := i and not dma_aux1;	{ lower AUX 1 }
  writeln(output,'write_control sends ',i or dma_aux1,' to dma reg.');
  port[pdma_dma] := i or dma_aux1;	{ raise AUX 1 }
end;

Function Plotter_status: integer;
begin
  plotter_status := port[pdma_b];
end;            { func plotter_status }

procedure step_head(direction : dir_type);
var
  porta : integer;
begin
  if direction = forw then
        porta := $047
  else
        porta := $045;
  write_control(porta);
  porta := porta or 8;  { raise ext step pulse }
  write_control(porta);
end;            { proc step_head }

{$ifdef debug}
procedure wait_count;
begin                   { don't wait on non-existant hardware }
end;
{$else }
procedure wait_count;
var
  I,J,timeout : integer;
begin
  timeout := 0;
  { timer ctrl reg SC1  SC0  RL1  RL0  M2  M1  M0  BCD }
  j := 0;
  repeat
    port[pdma_cntcsr] := $0E4;      { readback cmd, status, counter 1 }
    j := port[pdma_cnt1];         { read status byte }
    timeout := timeout + 1;
    if timeout >= 65535 then
      writeln(output,' timed out waiting for timer to go high.');
  until (j and 128) = 128;          { wait for counter to go high }
  timeout := 0;
  repeat
    port[pdma_cntcsr] := $0E4;      { readback cmd, status, counter 1 }
    j := port[pdma_cnt1];         { read status byte }
    timeout := timeout + 1;
    if timeout >= 65535 then
      writeln(output,' timed out waiting for timer to go low.');
  until (j and 128) = 0;            { wait for counter to go low }
end;    {proc wait_count}
{$endif }

function reference_head : boolean;
        { this procedure runs the head rapidly back to the index position }
begin
  repeat
    step_head(reverse);
    wait_count;
  until (plotter_status and 32 = 32);   { keep going until carriage index}
  for i := 1 to 50 do begin     { continue backward 50 more steps }
    step_head(reverse);
    wait_count;
 end;           { for 1 to 50 }
        { now, step forward slowly until you find both carriage index and stepper index }
  I:= 1;
  repeat
    step_head(forw);
    for j:= 1 to 8 do
      wait_count;
    i:=i+1;
  until (plotter_status and 96 = 96) or (i > 55);
  if i > 55 then begin
    writeln(output,'Can''t find carriage index position.');
    reference_head := false;
  end
  else
    reference_head := true;
end;             { function reference_head }

procedure init;
var
  i,j : integer;
begin
  writeln(output,'Init proc');
  writeln(output,'heap memory available ',MemAvail);
  writeln(output,'largest heap chunk available ',MaxAvail);
  num_beams := 1;
  flash_head := nil;
  flash_tail := nil;
  line_head := nil;
  line_tail := nil;
  for i := 0 to max_aperture do { mark all aperture codes as unused }
    dcode_array[i] := nil;
  border := 250;               { set a .25 inch border }
  with main_window do begin
        xmin :=  10000;
        xmax := -10000;
        ymin :=  10000;
        ymax := -10000;
  end;
  line_mask[0] := 1;    { construct line masks for 8-laser unit }
  line_mask[1] := 2;
  line_mask[2] := 4;
  line_mask[3] := 8;
  line_mask[4] := 16;
  line_mask[5] := 32;
  line_mask[6] := 64;
  line_mask[7] := 128;
  writeln(output,'Initializing PDMA card.');
  port[pdma_dma] := dma_adir + dma_aux1;	{ set a=output, aux1 high }
  port[pdma_int] := int_src0 + int_level * 16;	{ set int src to 8237 TC and int level }

  port[pdma_cntcsr] := 3 * ctr_rl0 + 2 * ctr_m0;	{ set ctr 0 to rate gen, and load both bytes }
  port[pdma_cnt0] := lo(20000);
  port[pdma_cnt0] := hi(20000);
  port[pdma_cntcsr] := 1*ctr_sc0+ 3 * ctr_rl0 + 2 * ctr_m0; {set ctr 1 to rate gen, load both bytes}
  port[pdma_cnt1] := lo(2);
  port[pdma_cnt1] := hi(2);
  apertures_loaded := false;
  gerber_loaded := false;
  j := port[pdma_base+3] or 114;   {dsbl int }
  port[pdma_base+3] := j and 127;   {dsbl int }

  {  if reference_head then begin }
  if true then begin      { fake moving head to reference position }
        { carriage now at index position, motor running }
    write_control(cr_mot_start+cr_step_forw+cr_1_step);     { start motor }
    write_control(cr_mot_start+cr_step_forw+cr_1_step);     { make sure }
    writeln(output,'Waiting for motor to speed up.');
    for i := 0 to 1000 do
      wait_count;
    j := port[pdma_int] or 114;   {dsbl int }
    port[pdma_int] := j and 127;   {dsbl int }
{    if (plotter_status and 1) <> 0 then
      writeln(output,' Speed servo lock lost.'); }
    band_complete := true;         { prevent check_band from hanging first
                                     time through }
    intercom_semaphore := false;     { mark no message pending }
    flash_count := 0;	                { zero statistics accumulators }
    draw_count_point := 0;
    draw_count_vert := 0;
    draw_count_horiz := 0;
    draw_count_diag := 0;
  end;
end;                                    { proc init }

{ the following function gets the linear address of a Global Descriptor }
function GetSegBaseAddr(Selector:word):longint; assembler;
asm
	mov	ax,0006h
	mov	bx,[Selector]
	int	31h
        jnc     @@1             { carry set if error }
        mov     dx,0ffffh       { error code in ax, impossible addr in hi part }
        jmp     @@2
@@1:	mov	ax,dx           { return lower part of result }
	mov	dx,cx           { return upper part }
@@2:
end;

{ the following function gets the descriptor from a Selector }
procedure GetDescriptor(Selector:word; var descrip: descrip_type);
{ assembler;}
var
  a_seg, a_ofs : word;
begin
  a_ofs := ofs(descrip);
  asm
	mov	ax,000Bh
	mov	bx,[Selector]
        les     cx,descrip
        mov     di,a_ofs
	int	31h
  end;
end;

{ the following function locks a DMA segemnt, and returns the Physical Address }
function LockDMABufReg(var my_DDS : DDS_Rec): integer;
{ flags in DX : bit 1      if data should be copied to buffer
                bit 2      disables alloc if region not contig or crosses phys boundary
                bit 3      disable auto remap attempt
                bit 4      region must not cross 64K phys boundary
                bit 5      region must not cross 128K phys boundary }

{ assembler;}
var
  a_seg, a_ofs : word;
begin
  a_ofs := ofs(my_DDS);
  asm
	mov	ax,8103h
	mov	dx,0012h	{ disab if not contig, don't cross 64K }
        les     cx,my_DDS
        mov     di,a_ofs
	int	4bh
        mov     @result,ax
        jc     @error           { leave error code in AL }
        mov    ax,0             { no error, clear AX }
        mov    @result,ax
  @error:
  end;
end;

{ the following function unlocks a DMA segemnt }
function unlockDMABufReg(var my_DDS : DDS_Rec): integer;
{ flags in DX : bit 1      if data should be copied out of buffer }

{ assembler;}
var
  a_seg, a_ofs : word;
begin
  a_ofs := ofs(my_DDS);
  asm
	mov	ax,8104h
	mov	dx,0
        les     cx,my_DDS
        mov     di,a_ofs
	int	4bh
        mov     @result,ax
        jc     @error           { leave error code in AL }
        mov    ax,0             { no error, clear AX }
        mov    @result,ax
  @error:
  end;
end;

procedure VDSGetVersion( var block : VDS_Block_rec);

{ assembler;}
begin
  asm
	mov	ax,8102h
	mov	dx,00h          { all flags must be 0 }
	int	4bh
        push    bx
        les     bx,block
        mov     es:[bx],ax      { maj_min }
        pop     ax
        mov     es:[bx+2],ax    { prod }
        mov     es:[bx+4],cx    { rev }
        mov     es:[bx+6],di    { dma_lo }
        mov     es:[bx+8],si    { dma_hi }
        mov     es:[bx+10],dx   { flags }
        mov     ax,0            { 0 means no error }
        mov     es:[bx+12],ax   { error }
        jnc      @error         { done if no error }
        mov     ax,-1           { -1 means an error }
        mov     es:[bx+12],ax
  @error:
  end;
end;

{var RMPtr:pointer;
    MySelector:THandle;
begin
 MySelector:=GlobalAlloc(GHND,32000);
 RMPtr:=ptr(GetSegBaseAddr(MySelector) shr 4,0);
 {RMPtr is now a real mode pointer}

procedure wrt_hex(n:longint; len:integer);
var
 i : integer;
 hi,q : longint;
begin
{    if len=2 then n:=n and $000000FF;
    if len=3 then n:=n and $00000FFF;
    if len=4 then n:=n and $0000FFFF;
    if len=5 then n:=n and $000FFFFF;
    if len=6 then n:=n and $00FFFFFF;
    if len=7 then n:=n and $0FFFFFFF;
    if len=8 then n:=n and $FFFFFFFF;
    hi := 1;}
{    for i := 1 to len-1 do
        hi := hi * 16;
    for i := 1 to len do begin
        q:= n div hi;
        n := n and $00ff;}
        for i := 1 to len do begin
        q := (n shr ((len - i) * 4)) and $00F;
        if q < 10 then
            write(output,chr(48+q))
        else
            write(output,chr(55+q));
{        n := n-q*hi;
        hi := hi div 16;}
    end;
end; { proc wrt_hex }

procedure start_dma(line_ptr : band_ptr_type; LineHandle : THandle);
{ note that this routine is called both by send_band (at user level) and by
	plot_line_isr (at interrupt service level).  Although this routine should
	be concurrently reusable (I think), the way this program is constructed,
        it should never be called upon to do so . }
{ It is pretty important to treat this routine as a CRITICAL SECTION, however,
	because it is brutally frank in how it treats the 8237 DMA controller,
	and could cause some gross interference if two routines are setting
	8237 registers concurrently. }
var
  i,j          : integer;
  ofset,segmnt : word;          { components of pointer }
  idsb,dma_pageaddr,dma_bytcnt : integer;
  dma_addr     : longint;
  itext        : string;
  descrip : descrip_type;
  a_dds : dds_rec;
  status : word;
  VDS_Block : VDS_Block_rec;
  dma_linear : longint;
  available : boolean;
begin
  available := true;
{ probably need to disable ALL interrupts here }
  j := port[pdma_int] or 114;   {dsbl int from PDMA card}
  port[pdma_int] := j and 127;   {dsbl int }
  port[$20] := $20;        { send EOI code to 8259 int controller }
  i := i * 2;              { delay }
  inline($FB);             { STI instruction }
  { convert descriptor fields to DDS fields }
    {  GetDescriptor(LineHandle,descrip);  }
      dma_linear := GetSegBaseAddr(seg(line_ptr));
      this_line := Line_ptr;  {save offset in a global so buf can be unlocked when done}
      a_dds.Region_Size := descrip[0] and $0000FFFF;    { seg limit 0..15 }
      if (descrip[1] and $00800000) <> 0 then
        a_DDS.Region_Size := a_DDS.Region_Size shl 12;  { seg limit * granularity }
      a_DDS.Selector := 0;
      a_DDS.Offset   := DMA_linear;
      status := 1234;
      repeat
        status := LockDMABufReg(a_DDS);
        if (status <> 0) then begin
          write(output,' LockDMA Buffer Region error = ');
          wrt_hex(status,2);
          writeln(output);
        end;
      until status <> 6;        { loop until buffer free }
      write(output,'Physical Address = ');
      wrt_hex(a_DDS.Phys_Addr,8);
      write(output,' buf id = ',a_DDS.buffer_ID);
      writeln(output);
     { now cause DMA transfer to plotter }
  port[dc_bptr] := 0;              { force 8237 byte pointer to known value }
  if dma_level = 1 then
    idsb := 5		{ get disable code for appropriate DMA level }
  else
    idsb := 7;
  port[dc_mask] := idsb;	{ disable DMA }
  port[dc_mode] := dma_level + 8 + 64;	{ ch #, 'read' mem, no auto init, addr incr, single mode }
  dma_pageaddr := segmnt div 4096;	{ calc value for DMA page reg }
  dma_addr := segmnt * 16;		{ build lower 16 bits of phys addr }
  dma_addr := dma_addr + ofset;
  dma_addr := dma_addr and 65535;	{ just keep lowest 16 bits }
  dma_bytcnt := line_length - 1;		{ 8237 must be loaded with bytecount - 1 }
  if dma_level = 1 then begin
    port[dc_page1] := dma_pageaddr;	{ set the page addr }
    port[dc_bc1] := lo(dma_bytcnt);	{ set the byte count }
    port[dc_bc1] := hi(dma_bytcnt);
    port[dc_br1] := lo(dma_addr);	{ set the lower 16 bit addr }
    port[dc_br1] := hi(dma_addr);
    port[pdma_dma] := port[pdma_dma] or dma_enab;	{ set DMA enable bit on intf card }
  end else begin
    port[dc_page3] := dma_pageaddr;
    port[dc_bc3] := lo(dma_bytcnt);
    port[dc_bc3] := hi(dma_bytcnt);
    port[dc_br3] := lo(dma_addr);
    port[dc_br3] := hi(dma_addr);
    port[pdma_dma] := port[pdma_dma] or dma_enab + dma_lev3;
  end;
  port[dc_mask] := dma_level;		{ enable DMA mask for this level }
  i := port[pdma_dma];			{ trigger first DMA request }
  port[pdma_dma] := i or dma_aux2;	{ raise AUX 2 }
  j := j + i;				{ short delay }
  port[pdma_dma] := i;			{ lower AUX 2 }
  j := port[pdma_int or 128];   {enable int on PDMA card}
  port[pdma_int] := j;
end;		{ proc start_dma }

procedure plot_line_isr (Flags,CS,IP,AX,BX,CX,DX,SI,DI,DS,ES,BP : word); interrupt;
var
  i,j          : integer;
  line_ptr     : band_ptr_type; {pointer to first byte of plot line }
  ofset,segmnt : word;          { components of pointer }
  idsb,dma_pageaddr,dma_bytcnt : integer;
  dma_addr     : longint;
  itext        : string;
  descrip : descrip_type;
  a_dds : dds_rec;
  status : word;
  VDS_Block : VDS_Block_rec;
  dma_linear : longint;
  LineHandle   : THandle;
  available : boolean;

begin
  {$ifdef intercom}
  str(plotting_line,intercom_text);
  intercom_text := 'ISR entered, b line ='+intercom_text;
  intercom_semaphore := true;
  {$endif}
  j := port[pdma_base+3] or 114;   {dsbl int }
  port[pdma_base+3] := j and 127;   {dsbl int }
  port[$20] := $20;        { send EOI code to 8259 int controller }
  i := i * 2;              { delay }
  inline($FB);             { STI instruction }
      dma_linear := GetSegBaseAddr(seg(This_Line));
   { convert descriptor fields to DDS fields }
      a_dds.Region_Size := descrip[0] and $0000FFFF;    { seg limit 0..15 }
      if (descrip[1] and $00800000) <> 0 then
        a_DDS.Region_Size := a_DDS.Region_Size shl 12;  { seg limit * granularity }
      a_DDS.Selector := 0;
      a_DDS.Offset   := DMA_linear;
      status := 1234;
      status := UnlockDMABufReg(a_DDS);
{ ***** no write calls allowed *****
      if (status <> 0) then begin
        write(output,' UnlockDMA Buffer Region error = ');
        wrt_hex(status,2);
        writeln(output);
      end;      }
  plotting_line := plotting_line - 1;  { bump to next line }
  if plotting_line < 0 then
    band_complete := true  { set completion semaphore for check_band }
  else begin
  if plotting_band = 1 then begin
      line_ptr := band1_array[plotting_line];
      LineHandle := band1_handle[plotting_line];
  end else begin
      line_ptr := band2_array[plotting_line];
      LineHandle := band1_handle[plotting_line];
  end;
  {start_dma(line_ptr,LineHandle);}
{ code inserted here is copy of code in start_dma, but made
	so that it shouldn't cause a problem as an ISR }
  available := true;
   { convert descriptor fields to DDS fields }
    {  GetDescriptor(LineHandle,descrip);  }
      dma_linear := GetSegBaseAddr(seg(line_ptr));
      this_line := Line_ptr;  {save offset in a global so buf can be unlocked when done}
      a_dds.Region_Size := descrip[0] and $0000FFFF;    { seg limit 0..15 }
      if (descrip[1] and $00800000) <> 0 then
        a_DDS.Region_Size := a_DDS.Region_Size shl 12;  { seg limit * granularity }
      a_DDS.Selector := 0;
      a_DDS.Offset   := DMA_linear;
      status := 1234;
      repeat
        status := LockDMABufReg(a_DDS);
{   ***** can't write in ISR *****
     if (status <> 0) then begin
          write(output,' LockDMA Buffer Region error = ');
          wrt_hex(status,2);
          writeln(output);
        end;}
      until status <> 6;        { loop until buffer free }
{      write(output,'Physical Address = ');
      wrt_hex(a_DDS.Phys_Addr,8);
      write(output,' buf id = ',a_DDS.buffer_ID);
      writeln(output);}
     { now cause DMA transfer to plotter }
  port[dc_bptr] := 0;              { force 8237 byte pointer to known value }
  if dma_level = 1 then
    idsb := 5		{ get disable code for appropriate DMA level }
  else
    idsb := 7;
  port[dc_mask] := idsb;	{ disable DMA }
  port[dc_mode] := dma_level + 8 + 64;	{ ch #, 'read' mem, no auto init, addr incr, single mode }
  dma_pageaddr := segmnt div 4096;	{ calc value for DMA page reg }
  dma_addr := segmnt * 16;		{ build lower 16 bits of phys addr }
  dma_addr := dma_addr + ofset;
  dma_addr := dma_addr and 65535;	{ just keep lowest 16 bits }
  dma_bytcnt := line_length - 1;		{ 8237 must be loaded with bytecount - 1 }
  if dma_level = 1 then begin
    port[dc_page1] := dma_pageaddr;	{ set the page addr }
    port[dc_bc1] := lo(dma_bytcnt);	{ set the byte count }
    port[dc_bc1] := hi(dma_bytcnt);
    port[dc_br1] := lo(dma_addr);	{ set the lower 16 bit addr }
    port[dc_br1] := hi(dma_addr);
    port[pdma_dma] := port[pdma_dma] or dma_enab;	{ set DMA enable bit on intf card }
  end else begin
    port[dc_page3] := dma_pageaddr;
    port[dc_bc3] := lo(dma_bytcnt);
    port[dc_bc3] := hi(dma_bytcnt);
    port[dc_br3] := lo(dma_addr);
    port[dc_br3] := hi(dma_addr);
    port[pdma_dma] := port[pdma_dma] or dma_enab + dma_lev3;
  end;
  port[dc_mask] := dma_level;		{ enable DMA mask for this level }
  i := port[pdma_dma];			{ trigger first DMA request }
  port[pdma_dma] := i or dma_aux2;	{ raise AUX 2 }
  port[pdma_dma] := i;			{ lower AUX 2 }
  j := port[pdma_int or 128];   {enable int on PDMA card}
  port[pdma_int] := j;

{ end of modified Start_DMA code}

  end;          { if plot_line }
  {$ifdef intercom}
  intercom_text := 'ISR completed';
  intercom_semaphore := true;
  {$endif}
end;            { proc plot_line_isr }

procedure define_bands;
{ calculates best arrangement for bands, and offsets }
var
  i,j : integer;
  band_buf,dma_physical,dma_end,add_on,dma_seg_l,dma_ofs_l : longint;
  dma_seg,dma_ofs : word;
  a_pointer : ptrrec;
  globalhandle : THandle;
  pmptr : pointer;
  descrip : descrip_type;
  a_dds : dds_rec;
  status : word;
  VDS_Block : VDS_Block_rec;
  dma_linear : longint;
begin
  with main_window do begin
    line_length := xmax - xmin + 2* border;                     { add border }
    writeln(output,' Plot line length is ',line_length);
    x_offset := xmin - border;
    y_offset := ymax + border;                  { plot from top down }
    writeln(output,' X, Y offsets = ',x_offset:8,y_offset:8);
    lines_band := memory div (2 * line_length);
    if lines_band > max_band_length then lines_band := max_band_length;
    writeln(output,'There will be ',lines_band,' lines per rasterizing band.');
    band_width := lines_band * num_beams;
    writeln(output,'This equals ',band_width,' mils per rasterizing band.');
    band_buf := line_length;
    band_buf := band_buf * 2 * lines_band;
    if (Byte(ptr($0040,$007b)^) and $020) = 0 then
      writeln(output,' WARNING - VDS not operating !');
    VDS_Block.maj_min := 0;
    VDS_Block.prod := 0;
    VDS_Block.rev := 0;
    VDS_Block.dma_hi := 0;
    VDS_Block.dma_lo := 0;
    VDS_Block.flags := 0;
    VDS_Block.error := 0;
    VDSGetVersion(VDS_Block);
    write(output,'VDS block - maj, min = ');
    wrt_hex(VDS_Block.maj_min,4);
    writeln(output);
    write(output,'    Prod #, rev      = ');
    wrt_hex(VDS_Block.prod,4);
    write(output,'    ');
    wrt_hex(VDS_Block.rev,4);
    writeln(output);
    write(output,'    Max DMA buf Size (hex)= ');
    wrt_hex(VDS_Block.DMA_hi,4);
    write(output,'    ');
    wrt_hex(VDS_Block.DMA_lo,4);
    writeln(output);
    write(output,'    Flags/error stat = ');
    wrt_hex(VDS_Block.flags,4);
    write(output,'    ');
    wrt_hex(VDS_Block.error,4);
    writeln(output);

    writeln(output,'Allocating ',band_buf,
                   ' Bytes rasterizer band memory now.');
    for i := 0 to lines_band-1 do begin
        { alloc a block of mem, return handle }
      globalhandle := GlobalAlloc(gmem_Fixed,line_length);
      band1_handle[i] := globalhandle;
        { lock the memory block, conv addr to pointer }
      if globalhandle <> 0 then
        band1_array[i] := GlobalLock(globalhandle);
 {     write(output,'initial try - band1_array[',i,'] => ');
      wrt_hex(seg(band1_array[i]^),4);
      write(output,':');
      wrt_hex(ofs(band1_array[i]^),4);  
      writeln(output);
      write(output,'GlobalHandle => ');
      wrt_hex(globalhandle,8);
      writeln(output);  }
      { now check to see if this line buffer will cause DMA page wraparound }
      {dma_linear := GetSegBaseAddr(band1_handle[i].Selector);}
      GetDescriptor(GlobalHandle,descrip);
      if band1_array[i] = nil then begin
        writeln(output,'getmem(band1_array[',i,'] failed.');
        return_mem;
      end;
    end;      { for }
    for i := 0 to lines_band-1 do begin
      globalhandle := GlobalAlloc(gmem_Fixed,line_length);
      band2_handle[i] := globalhandle;
      if globalhandle <> 0 then
        band2_array[i] := GlobalLock(globalhandle);
 {     write(output,'initial try - band2_array[',i,'] => ');
      wrt_hex(seg(band2_array[i]^),4);
      write(output,':');
      wrt_hex(ofs(band2_array[i]^),4);  
      writeln(output);
      write(output,'GlobalHandle => ');
      wrt_hex(globalhandle,8);
      writeln(output);  }
      { now check to see if this line buffer will cause DMA page wraparound }
{      dma_linear := GetSegBaseAddr(band2_array[i]); }
      GetDescriptor(GlobalHandle,descrip);
      if band2_array[i] = nil then begin
        writeln(output,'getmem(band2_array[',i,'] failed.');
        return_mem;
      end;
    end;                                        { for i := }
    getintvec(int_num,@int_bdma_save);  { save orig int vec }
    j := port[pdma_int] or 114;   {dsbl int }
    port[pdma_int] := j and 127;   {dsbl int }
    {$ifndef debug}
{ point Interrupt vector at int svc routine }
    setintvec(int_num,@plot_line_ISR);
    writeln(output,'interrupt vector set');
    j := 1;                          { create interrupt unmask bit }
    for i := 1 to int_level do 
      j := j * 2;
    j := not j;         { complement the bit }     
    i := port[int_mask];
    port[int_mask] := i and j;  { clear the mask bit for our int level }
{    port[pdma_int] := port[pdma_int] or int_enab;	{ set int enable bit }
    {$endif}
    writeln(output,'heap memory available ',MemAvail);
    writeln(output,'largest heap chunk available ',MaxAvail);
  end;                                                  { with }
end;                                                    { proc define_bands }

procedure send_band ( which_band : integer);
var
  j : integer;
  line_ptr     : band_ptr_type; {pointer to first byte of plot line }
  LineHandle   : THandle;
  ofset,segmnt : word;          { components of above pointer }
begin
  j := port[pdma_base+3] or 114;   {dsbl int }
  port[pdma_base+3] := j and 127;   {dsbl int }
    {$ifdef verbose}
    writeln(output,'send-band top,int ctl reg = ',j and 127);
    {$endif}
  plotting_band := which_band;  { leave global info for int service rtn }
  plotting_line := lines_band - 1;
  {$ifndef debug}
  band_complete := false;	{ preset semaphore for ISR to use, later }
  {$endif}
  if plotting_band = 1 then begin
      line_ptr := band1_array[plotting_line];
      LineHandle := band1_handle[plotting_line];
  end else begin
      line_ptr := band2_array[plotting_line];
      LineHandle := band2_handle[plotting_line];
  end;
  start_dma(line_ptr,LineHandle);
  j := port[pdma_base+3] or 128;   {enable int }
  port[pdma_base+3] := j;
  {$ifdef verbose}
  writeln(output,'send-band bot,int ctl reg = ',port[pdma_int]);
  {$endif}
end;       { proc send_band }

procedure check_band (which_band : integer);
var
  i : integer;
begin
  writeln(output,'Waiting on band ',which_band,' to finish plotting.');
  i := 0;
  repeat                { wait for semaphore to be set }
    if intercom_semaphore then begin
      writeln(output,intercom_text);
      intercom_semaphore := false;
    end;
    {$ifdef intcom}
    i := i +1;
    if i > 1000 then begin
      writeln(output,'still waiting.');
      i := 0;
    end;
    {$endif}
  until band_complete;
  {$ifdef intcom}
  writeln(output,'band_complete was set.');
  {$endif}
end;

procedure return_mem;
var
  i,idsb : integer;
begin
  stop_motors;
    j := port[pdma_int] or 114;   {dsbl int on PDMA card}
    port[pdma_int] := j and 127;   {dsbl int }
    j := 1;                          { create interrupt mask bit }
    for i := 1 to int_level do 
      j := j * 2;
    i := port[int_mask];
    port[int_mask] := i or j;  { set the mask bit for our int level }
    if dma_level = 1 then
      idsb := 5		{ get disable code for appropriate DMA level }
    else
      idsb := 7;
    port[dc_mask] := idsb;	{ disable DMA }
  { free band buffers in reverse order of creating them }
  i := lines_band-1;
{  while i >= 0  do begin
    freemem(band2_array[i],line_length);
    i := i -1;
  end;
  while i >= 0  do begin
    freemem(band1_array[i],line_length);
    i := i -1;
  end;   }
{  setintvec(int_level,@int_bdma_save);  { put old int vec back }
 {  halt;     }
end;

procedure stop_motors;
begin
  write_control(0);
end;

end.		{ unit photodrv }


