unit photoman;

interface

procedure clear_band (xhich_band : integer);
Function read_aperture(var filevar : text) : integer;
Function read_gerber(var filevar : text) : integer;
procedure make_plot;

implementation
uses photogbl,photodrv,WinCrt;

{$I photodef.pas}


procedure clear_band (xhich_band : integer);
{ clears an entire band to appropriate blank condition for positive or negative }
var
  i,j,fill : integer;
begin
  if pos_neg = (positive) then
        fill := 0
  else
        fill := 255;
{  writeln(output,'clear_band, which_band, fill ',which_band:6,fill:6);
  writeln(output,'lines_band, line_length ',lines_band:6,line_length:6);}
  if which_band = 1 then begin
    for i := 0 to lines_band-1 do begin
      for j := 0 to line_length-1 do
        band1_array[i]^[j] := fill;
    end;
  end;
  if which_band = 2 then begin
    for i := 0 to lines_band-1 do begin
      for j := 0 to line_length-1 do
        band2_array[i]^[j] := fill;
    end;
  end;
end;                            { proc clear_band }


{*section-following procs for building database of drawing primitives }

procedure create_draw (old_x,old_y,x,y,d_code_num : integer);
var
  radius : integer;
  line_ptr : line_ptr_type;
  d_code   : dcode_ptr_type;
begin
  if (d_code_num > max_aperture) or (dcode_array[d_code_num] = nil) then
    writeln(output,' Unassigned or Bad aperture # ',d_code_num,' in create_draw.')
  else begin
    {$ifdef verbose}
        writeln(output,'Create_draw ',old_x:7,old_y:7,x:7,y:7,'  D',d_code_num);
    {$endif}
    d_code := dcode_array[d_code_num];
    new(line_ptr);                      { creates the record }
    if line_ptr = nil then begin
        writeln(output,'Memory exhausted in create_draw.');
        return_mem;
    end else begin
      if line_head = nil then line_head := line_ptr;
      if line_tail <> nil then line_tail^.next_line := line_ptr;
      line_tail := line_ptr;
      line_ptr^.x1 := old_x;
      line_ptr^.y1 := old_y;
      line_ptr^.x2 := x;
      line_ptr^.y2 := y;
      line_ptr^.this_apt := d_code;
      line_ptr^.width := d_code^.x_size;
      line_ptr^.next_line := nil;
    { maintain x,y min and max }
      radius := d_code^.x_radius;
      with main_window do begin
        if xmin > old_x - radius then xmin := old_x - radius;
        if xmin >       x - radius then xmin :=       x - radius;
        if ymin > old_y - radius then ymin := old_y - radius;
        if ymin >       y - radius then ymin :=       y - radius;
        if xmax < old_x + radius then xmax := old_x + radius;
        if xmax <       x + radius then xmax :=       x + radius;
        if ymax < old_y + radius then ymax := old_y + radius;
        if ymax <       y + radius then ymax :=       y + radius;
      end;                      { with }
    {$ifdef verbose}
       with main_window do
        writeln(output,'Create_draw , xmin,xmax,ymin, ymax ',
                xmin:8,xmax:8,ymin:8,ymax:8);
   {$endif}
    end;                        { if line_ptr = nil }
    if (y = old_y) and (x = old_x) then
      draw_count_point := draw_count_point + 1
    else begin
      if (y <> old_y) and (x <> old_x) then
        draw_count_diag := draw_count_diag + 1
      else begin
        if y = old_y then
          draw_count_vert := draw_count_vert + 1;
        if x = old_x then
          draw_count_horiz := draw_count_horiz + 1;
      end;
    end;
  end;                          { if d_code_num }
end;                    { proc create_draw }

procedure create_flash (x,y,d_code_num : integer);
var
  radius : integer;
  flash : flash_ptr_type;
  d_code : dcode_ptr_type;
begin
  if (d_code_num > max_aperture) or (dcode_array[d_code_num] = nil) then
    writeln(output,'Undefined or Bad aperture # ',d_code_num,' in create_flash.')
  else begin
    {$ifdef verbose}
        writeln(output,'Create_flash ',x:7,y:7,'  D',d_code_num);
    {$endif}
    new(flash);                         { creates the record }
    if flash = nil then begin
        writeln(output,'Memory exhausted in create_flash.');
        return_mem;
    end else begin
      d_code := dcode_array[d_code_num];
      if flash_head = nil then flash_head := flash;
      if flash_tail <> nil then flash_tail^.next_flash := flash;  { link this to previous last node }
      flash_tail := flash;
      flash^.x := x;
      flash^.y := y;
      flash^.this_apt := d_code;
      flash^.next_flash := nil;
      { maintain x,y min and max }
      radius := d_code^.x_radius;
      with main_window do begin
        if xmin >       x - radius then xmin :=       x - radius;
        if ymin >       y - radius then ymin :=       y - radius;
        if xmax <       x + radius then xmax :=       x + radius;
        if ymax <       y + radius then ymax :=       y + radius;
      end;                              { with }
    end;                                        { if flash = nil }
    flash_count := flash_count + 1;
  end;                          { if d_code_num }
end;                            { proc create_flash }



{ *section - following procedures for manipulating aperture templates }

procedure ras_fill_inner(left,right,line,bit,x_size,y_size : integer;
                this_dcode : dcode_ptr_type ;
                var bits_untyp );
type
    bits_type = array [0..maxint] of byte;
{ ***** prepare a typecast for UNTYPED variable parameter bits_untyp ***** }
{       bits_untyp is really a 2-dimensional array which contains a bit picture
        of an aperture }

var
  I,offset : integer;
  bit_mask, column,address : integer;
begin
{$ifdef aper_dump}
  writeln(output,'ras_fill l,r,line # ',left:5,right:5,line:5);
{$endif}
  if (line >= 0) and (line < this_dcode^.y_block_size) then begin
    if left < 0 then left := 0;
    if left > this_dcode^.x_block_size -1 then
      left:=this_dcode^.x_block_size -1;
    if right < 0 then right := 0;
    if right > this_dcode^.x_block_size -1 then
      right:=this_dcode^.x_block_size -1;
    if left > right then left := right;
    if num_beams = 1 then begin
      bit_mask := 1;
      column := line;
    end else begin              { assume 8 beams }
      bit_mask := line_mask[line mod 8];
      column := line div 8;
    end;                                { if num_beams }
    offset := column * x_size;
    if left = right then begin
      if bit = 1 then
      bits_type (bits_untyp) [left+offset] :=
        bits_type (bits_untyp) [left+offset] or bit_mask
      else
         bits_type (bits_untyp) [left+offset] :=
        bits_type (bits_untyp) [left+offset] and not bit_mask;
      end
   else begin
      if bit = 1 then
        for i := left to right do
        bits_type (bits_untyp) [i+offset] :=
          bits_type (bits_untyp) [i+offset] or bit_mask
      else
        for i := left to right do
        bits_type (bits_untyp) [i+offset] :=
          bits_type (bits_untyp) [i+offset] and not bit_mask;
    end;
  end;                          { if line >= 0 }
end;                            { proc ras_fill }


{ this is a dummy shell for ras_fill to allow the real ras_fill (ras_fill_inner) to
        receive a formal parameter of unknown length }
procedure ras_fill(left,right,line,bit : integer;
                   this_dcode : dcode_ptr_type);
{  fills one line in X axis from left to right with bit,
         into a d_code template }
var
  x_size, y_size : integer;
begin
  x_size := this_dcode^.x_size;
  y_size := this_dcode^.y_size;
  ras_fill_inner(left,right,line,bit,x_size,y_size,
        this_dcode,this_dcode^.bit_block^);
end;

procedure make_circle_aperture (radius : integer; D_code : dcode_ptr_type;
          bit : integer );
{this proc runs Bresenham's circle generator to make a circle of the
       desired radius in the specified aperture array }
var
        cx,cy,x,y,d,i,j,last_y : integer;
        aray_1 : array [0..1024] of integer;    { holds octant }
        aray_2 : array [0..2048] of integer;    { holds quadrant }
        middle : boolean;
begin
      cx := d_code^.x_size div 2;
      if d_code^.aperture = thermal then
        cy := d_code^.x_size div 2
      else
        cy := d_code^.y_size div 2;
    if not odd(d_code^.x_size) then  { fix for circles one pixel too big }
        radius := radius - 1;
    {$ifdef verbose}
       writeln(output,'Circle - x,y,radius =',cx:6,cy:6,radius:6);
    {$endif}
    middle:=false;
    i:=0; j:=0;
    x:=0;
    y:=radius;
    last_y:=radius;
    d:=3-2*radius;
    while x<y do begin
        aray_1[i]:=y;
        i:=i+1;
        if d<0 then
            d:=d+4*x+6
        else begin
            d:=d+4*(x-y)+10;
    {$ifdef dbug_circle}
            writeln(output,'point ',x:6,y:6);
    {$endif}
            ras_fill(cx-x,cx+x,cy+y,bit,d_code);
            last_y:=y;
            y:=y-1;
            aray_2[j]:=x;
            j:=j+1;
        end;
        x:=x+1;
    end;        {while}
    if x=y then begin
        middle:=true;
    {$ifdef dbug_circle}
        writeln(output,'point ',x:6,y:6);
    {$endif}
        ras_fill(cx-x,cx+x,cy+y,bit,d_code);
        last_y:=y;
        aray_2[j]:=x;
        j:=j+1;
    end;
    i:=i-1;
    while i>=0 do begin         { reflect first octant }
        y:=y-1;
        if y<last_y-1 then begin
    {$ifdef dbug_circle}
            writeln(output,'point ',aray_1[i]:6,(y+1):6);
    {$endif}
            ras_fill(cx-aray_1[i],cx+aray_1[i],cy+y+1,bit,d_code);
            aray_2[j]:=aray_1[i];
            j:=j+1;
        end;
    {$ifdef dbug_circle}
        writeln(output,'point ',aray_1[i]:6,y:6);
    {$endif}
        ras_fill(cx-aray_1[i],cx+aray_1[i],cy+y,bit,d_code);
        last_y:=y;
        aray_2[j]:=aray_1[i];
        j:=j+1;
        i:=i-1;
    end;
    j:=j-2;
    if not middle then j:=j-2;
    while j>=0 do begin         { reflect first quadrant }
        y:=y-1;
    {$ifdef dbug_circle}
        writeln(output,'point ',aray_2[j]:6,y:6);
    {$endif}
        ras_fill(cx-aray_2[j],cx+aray_2[j],cy+y,bit,d_code);
        j:=j-1;
    end;
end;            { proc make_circle_aperture }


procedure make_oval_aperture (x_size , y_size : integer; D_code : dcode_ptr_type );
var
        cx,cy, radius,bit,rect : integer;
        x,y,d,i,j,k,last_y : integer;
        aray_1 : array [0..1024] of integer;    { holds octant }
        aray_2 : array [0..2048] of integer;    { holds quadrant }
        middle,horizontal : boolean;
begin
  cx := d_code^.x_center;
  cy := d_code^.y_center;
  if x_size > y_size then begin
      horizontal := true;                       { major dimension is X axis }
      radius := d_code^.y_size div 2;
      rect := d_code^.x_size div 2 - radius;
    end else begin
      horizontal := false;                      { major dimension is Y axis }
      radius := d_code^.x_size div 2;
      rect := d_code^.y_size div 2 - radius;
    end;
            {$ifdef verbose}
    writeln(output,'Radius was ',radius);
    {$endif}
    if not odd(radius) then                     { fix for circles one pixel too big }
        radius := radius - 1;
            {$ifdef verbose}
    writeln(output,'Radius is ',radius);
        writeln(output,'Make_oval_aperture - x,y,radius =',cx:6,cy:6,radius:6);
    {$endif}
    bit := 1;
    middle:=false;
    i:=0; j:=0;
    x:=0;
    y:=radius;
    last_y:=radius;
    d:=3-2*radius;
    while x<y do begin
        aray_1[i]:=y;
        i:=i+1;
        if d<0 then
            d:=d+4*x+6
        else begin
            d:=d+4*(x-y)+10;
{           writeln(output,'point ',x:6,y:6);}
            if horizontal then
              ras_fill(cx-x-rect,cx+x+rect,cy+y,1,d_code)
            else
              ras_fill(cx-x,cx+x,cy+y+rect-1,1,d_code);
            {$ifdef verbose}
            writeln(output,' x<y ras_fill from ',cx-x,' to ',cx+x,' line ',cy+y+rect-1);
    {$endif}
            last_y:=y;
            y:=y-1;
            aray_2[j]:=x;
            j:=j+1;
        end;
        x:=x+1;
    end;        {while}
    if x=y then begin
        middle:=true;
{       writeln(output,'point ',x:6,y:6);}
        if horizontal then
              ras_fill(cx-x-rect,cx+x+rect,cy+y,1,d_code)
        else
                ras_fill(cx-x,cx+x,cy+y+rect,1,d_code);
            {$ifdef verbose}
        writeln(output,' x=y ras_fill from ',cx-x,' to ',cx+x,' line ',cy+y+rect);
    {$endif}
        last_y:=y;
        aray_2[j]:=x;
        j:=j+1;
    end;
    i:=i-1;
    if not horizontal then              { make center rectangle }
      for k := rect downto -rect-1 do
        ras_fill(cx-radius,cx+radius,cy+k,1,d_code);
    while i>=0 do begin         { reflect first octant }
        y:=y-1;
        if y<last_y-1 then begin
{           writeln(output,'point ',aray_1[i]:6,(y+1):6);}
            if horizontal then
              ras_fill(cx-aray_1[i]-rect,cx+aray_1[i]+rect,cy+y+1,1,d_code)
            else
                ras_fill(cx-aray_1[i],cx+aray_1[i],cy+y-rect+1,1,d_code);
            aray_2[j]:=aray_1[i];
            j:=j+1;
        end;
{       writeln(output,'point ',aray_1[i]:6,y:6);}
        if horizontal then
              ras_fill(cx-aray_1[i]-rect,cx+aray_1[i]+rect,cy+y,1,d_code)
        else
                ras_fill(cx-aray_1[i],cx+aray_1[i],cy+y+rect,1,d_code);
        last_y:=y;
        aray_2[j]:=aray_1[i];
        j:=j+1;
        i:=i-1;
    end;
    j:=j-2;
    if not middle then j:=j-2;
    while j>=0 do begin         { reflect first quadrant }
        y:=y-1;
{       writeln(output,'point ',aray_2[j]:6,y:6);}
        if horizontal then
              ras_fill(cx-aray_2[j]-rect,cx+aray_2[j]+rect,cy+y,1,d_code)
        else
                ras_fill(cx-aray_2[j],cx+aray_2[j],cy+y-rect,1,d_code);
        j:=j-1;
    end;
end;            { proc make_oval_aperture }
{$undef verbose}

procedure make_rect_aperture (x_size, y_size : integer; D_code : dcode_ptr_type );
var
    i : integer;
begin
    {$ifdef verbose}
        writeln(output,'Make_rect_aperture - x,y size =',x_size:6,y_size:6);
    {$endif}
  for i:=0 to y_size-1 do
    ras_fill(0,x_size-1,i,1,d_code);
end;


Procedure create_aperture (aperture : aperture_type;
                           d_code_num : integer;
                           x_size,y_size,hole_size : integer);
var
  x_block_size,y_block_size,i,j,ysz : integer;
  d_code : dcode_ptr_type;
  aray_ptr : ^bits_type;
  aray_size,xw,yw : word;
begin
  {$ifdef verbose}
        writeln(output,'Create_aperture D',d_code_num,'  ',x_size:4,y_size:4,hole_size:4);
  {$endif}
  x_block_size := x_size;
  if num_beams = 1 then
    if aperture = thermal then
      y_block_size := x_size
    else
      y_block_size := y_size
  else begin
    y_block_size := y_size div 8;
    if y_block_size mod 8 <> 0 then
      y_block_size := y_block_size + 1;
  end;
  if dcode_array[d_code_num] = nil then begin
    if (aperture = ellipse) and (x_size = y_size) then
        aperture := circle;
    new(dcode_array[d_code_num]);
    d_code := dcode_array[d_code_num];
      d_code^.aperture := aperture;
      d_code^.x_size := x_size;
      d_code^.y_size := y_size;
      d_code^.x_radius := x_size div 2;
      d_code^.y_radius := y_size div 2;
      d_code^.hole_size := hole_size;
      d_code^.x_block_size := x_block_size;
      d_code^.y_block_size := y_block_size;
      d_code^.x_center := x_size div 2;
      d_code^.d_num := d_code_num;
      if aperture = thermal then
        ysz := x_size { special for thermals }
      else
        ysz := y_size;
      if num_beams = 1 then
        d_code^.y_center := ysz div 2
      else begin
        d_code^.y_center := ysz div 16;
        d_code^.y_offset := (ysz div 2) mod 8;
      end;
    xw := x_size;       { convert to word }
    if aperture = thermal then
      yw := x_size  { thermals are always round, OD defined by x_size }
    else
      yw := y_size;
    aray_size := xw*yw;
 {   writeln(output,' getmem of ',aray_size,' bytes.'); }
    if aray_size > max_ap_area then
      writeln(output,' aperture D',d_code_num,' exceeds area ',
          'maximum for an aperture.')
    else begin

    { warning !  getmem apparently has a problem under TPW 1.5 when
    allocating a block of 40000 bytes - looks like you get 32668 ! }

    getmem(d_code^.bit_block,aray_size);  { allocate space for bitmap of aperture }
        { clear the bitmap before we start building the aperture bitmap }
    if  d_code^.bit_block = nil then
        writeln(output,' aperture D',d_code_num,' failed to allocate ',
          'bitmap memory.')
    else begin
    aray_ptr := ptr(ptrrec(d_code^.bit_block).seg,
               ptrrec(d_code^.bit_block).ofs);    { convert type here }
    for i := 0 to ysz-1 do
        for j := 0 to d_code^.x_size-1 do
          aray_ptr^[j+i*d_code^.x_size] := 0;
    case aperture of
      circle : begin
	           make_circle_aperture(x_size div 2, d_code, 1);
		     if (hole_size > 2) then  { punch hole in pad }
                   make_circle_aperture(hole_size div 2, d_code, 0);
		   end;
      oval   : make_oval_aperture(x_size, y_size,d_code);
      rectangle : make_rect_aperture(x_size, y_size, d_code);
      thermal : begin
        make_circle_aperture(x_size div 2,d_code,1); {outer circle }
        make_circle_aperture(y_size div 2,d_code,0); {cut inner circle }
        for i:=d_code^.x_center-hole_size div 2 to   {cut relief in sides }
              d_code^.x_center+hole_size div 2  do
                ras_fill(0,x_size-1,i,0,d_code);
        for i:= 0 to x_size-1 do                {cut relief in top & bot }
                ras_fill(d_code^.x_center-hole_size div 2,
                d_code^.x_center+hole_size div 2,i,0,d_code);
      end;      { thermal }
        { missing some routines here ***** }
    end;                         { case aperture }
    end;                         { if dcode^.bit_block = nil }
    end                          { if aray_size > }
  end else
    writeln(output,'trying to reuse aperture # D',d_code_num);
end;                            { proc create_aperture }

{ *section - procedures to read input files follow }

function getnum (var filevar : text; var achar : char) : longint;
var
    temp    : longint;
    eolflag : boolean;
begin
    temp := 0;
    repeat                      { skip leading blanks & tabs }
        read(filevar,achar);
    until (achar >' ') or eof(filevar);
    if not eof(filevar) then begin
      repeat
        if (achar<'0') or (achar>'9') then
            writeln(output,'error in getnum, char ="',achar,'"');
        temp := temp*10+ord(achar)-ord('0');
        if not eoln(filevar) then begin
            read(filevar,achar);
            eolflag := false;
        end
        else eolflag := true;
      until (achar<'0') or (achar>'9') or eolflag or eof(filevar);
    end;
    getnum := temp;
end;


Function read_aperture(var filevar : text) : integer;
{       Aperture list file format :
  D10             C             50             50            15                 F
dcode          type     x size     y size            hole size  flash code
D10 to      C = Circle                                  F = Flash
D99         O = Oval                                    D = Draw
            S = Rectangle ( & Square )                  B = Flash & Draw
            R = Rounded Rectangle
            E = Ellipse
            T = Thermal (X = outer diam, Y = gap width, Hole = inner diam ) }

var
  line_num, dnum : integer;
  achar, dchar : char;
  type_char,flash_char : char;
  aper_type : aperture_type;
  flash_code : flash_type;
  aperture_valid : boolean;
  x_size,y_size,hole_size : integer;
  d_code : dcode_ptr_type;
  aray_ptr : ^bits_type;

begin
  line_num :=1;
  while not eof(filevar) do begin
    repeat
      read(filevar,dchar);
    until dchar <> ' ';      { throw away any spaces at beginning of line }
    if (dchar <> 'd') and (dchar <> 'D') then
      writeln(output,'Bad D-code char >',dchar,'< in first char pos of line '
                          ,line_num,' of aperture list.');
    dnum := getnum(filevar,achar);
    if (dnum < 10) or (dnum > 99) then
      writeln(output,'Bad D-code number ',dnum,' in line ',
                          line_num,' of Aperture List.')
    else begin
      repeat
        read(filevar,type_char);
      until type_char <> ' ';      { throw away any leading spaces }
      aperture_valid := true;
      case type_char of
        'c','C' : aper_type := circle;
        'o','O' : aper_type := oval;
        's','S','r','R' : aper_type := rectangle;  { accept either Square or Rect }
        'm','M' : aper_type := round_rectangle;    { not implemented yet }
        'e','E' : aper_type := ellipse;            { not implemented yet }
        't','T' : aper_type := thermal;
      else begin
        writeln(output,'Bad aperture type >',type_char,'< in line ',line_num,
                ' of aperture list.');
        aperture_valid := false;
        end;                            { begin }
      end;                              { case type_char }
      repeat
        read(filevar,type_char);        { gobble rest of type name, if any }
      until type_char = ' ';
      x_size := getnum(filevar,achar);
      y_size := getnum(filevar,achar);
      hole_size := getnum(filevar,achar);
      repeat
        read(filevar,flash_char);
      until flash_char <> ' ';
      case flash_char of
        'f','F' : flash_code := flash_it;
        'd','D' : flash_code := draw_it;
        'b','B' : flash_code := both;
      else begin
        writeln(output,'Bad flash/draw code ',flash_char,
                ' in line ',line_num,' of aperture list.');
        aperture_valid := false;
        end;            { begin }
      end;              { case }
      readln(filevar);  { gobble end of line }
      if aperture_valid then
        create_aperture(aper_type,dnum,x_size,y_size,hole_size);
    end;                        { if (dnum < 10 ... }
    line_num := line_num + 1;
  end;                  { while }
  read_aperture := line_num;
  {$ifdef aper_dump}
  writeln(output,' APERTURE LIST ');
  for dnum:= 0 to max_aperture do begin
    d_code := dcode_array[dnum];
    if d_code <> nil then begin
      aray_ptr := ptr(ptrrec(d_code^.bit_block).seg,
               ptrrec(d_code^.bit_block).ofs);    { convert type here }
      aray_ptr := ptr(ptrrec(aray_ptr).seg,ptrrec(aray_ptr).ofs);
      writeln(output,' Aperture D',dnum:2);
      for y_size := 0 to d_code^.y_size-1 do begin
        write(output,y_size:3,' >');
        for x_size := 0 to d_code^.x_size-1 do
          if aray_ptr^[x_size+y_size*d_code^.x_size] <> 0 then
        write(output,'*')
          else
        write(output,' ');
        writeln(output,'<');
      end;              { for x_size }
    end;                        { if d_code <> nil }
  end;                  { for dnum := 0 }
  {$endif}
end;                            { proc read_aperture }

Function read_gerber(var filevar : text) : integer;
var
  line_num,d_num,pen_code,x,y,old_x,old_y,d_code : integer;
  achar : char;
  f_flag,s_flag,l_flag,a_flag,end_flag : boolean;
  x_format,y_format,x_fmt_rt,y_fmt_rt,a : integer;
  l_x,l_y                               : longint;
begin
  f_flag := false;
  s_flag := false;
  l_flag := false;
  a_flag := false;
  end_flag := false;
  line_num := 1;
  while (not eof(filevar)) and (not end_flag) do begin
    read(filevar,achar);
    {$ifdef verbose}
      writeln(output,'Read_Gerber parsing ',achar);
    {$endif}
        { * at beginning of line means setup line }
    case achar of
      '*','%' : begin
        while not eoln(filevar) do begin
          read(filevar,achar);
          case achar of
          '*' : {ignore };
          '%' : {ignore };
          'f','F' : f_flag := true;
          's','S' : s_flag := true;
          'l','L' : l_flag := true;
          'a','A' : a_flag := true;
          'x','X' : begin
                x_format := getnum(filevar,achar);
		      writeln(output,'X format is ',x_format);
                a := x_format div 10;
			writeln(output,'a is ',a);
                x_fmt_rt := x_format - a * 10;
			writeln(output,'x_fmt_rt is ',x_fmt_rt);
                if (achar = 'y') or (achar = 'Y') then begin
                  y_format := getnum(filevar,achar);
                  a := y_format div 10;
                  y_fmt_rt := y_format - a * 10;
			writeln(output,'y_fmt_rt is ',y_fmt_rt);
                end;                   {if  begin }
              end;                     { 'x' : begin }
          'y','y' : begin
                y_format := getnum(filevar,achar);
                a := y_format div 10;
                y_fmt_rt := y_format - a * 10;
		    writeln(output,'y_fmt_rt is ',y_fmt_rt);
              end;
        else writeln(output,'Unknown character in Gerber setup line, ',achar);
          end;          { case achar (for lines beginning with a * }
        end;            { while not eoln }
        end;            {  case '*' : begin }
    's','S' : writeln(output,' Don''t know what to do with S');
    'g','G' : writeln(output,' Don''t know what to do with G');
    'd','D' : begin
        d_num := getnum(filevar,achar);
        if d_num < 4 then begin
        { this is pen up / pen down command }
          pen_code := d_num;
        {$ifdef verbose}
          writeln(output,'Read_Gerber pen code = ',pen_code);
        {$endif}
          case pen_code of
            1 : begin
                  create_draw (old_x,old_y,x,y,d_code);
                  old_x := x;
                  old_y := y;
                end;
            2 : begin                   { 'pen up' move }
                  old_x := x;
                  old_y := y;
                end;
            3 : create_flash(x,y,d_code);
          end;          { case pen_code }
        end else begin
        { select an aperture }
          if d_num > max_aperture then
            writeln(output,'Specified aperture number ',d_num,' too high in line ',
                line_num,' of Gerber file.')
          else
            d_code := d_num;
            {$ifdef verbose}
              writeln(output,'Read_Gerber aperture = ',d_num);
            {$endif}
          end;
          end;          { case D }
    'x','X' : begin
        {$ifdef verbose }
            writeln(output,' read_gerber, getting X coord.');
        {$endif}
          l_x := getnum(filevar,achar);
        {$ifdef verbose }
            writeln(output,' read_gerber, X coord = ',x,' term char =>',achar,'<');
        {$endif}
          case x_fmt_rt of
            2 : l_x := l_x * 10;
            3 : ;
            4 : L_x := l_x div 10;
            5 : l_x := l_x div 100;
            6 : l_x := l_x div 1000;
          end;          { case x_fmt_rt }
          x := l_x;
          if (achar = 'd') or (achar = 'D') then begin
            d_num := getnum(filevar,achar);
            if d_num >= 4 then
              writeln(output,'unexpected pen change in coordinate line # ',
                  line_num)
            else begin
            pen_code := d_num;
            {$ifdef verbose}
              writeln(output,'Read_Gerber pen code = ',pen_code);
            {$endif}
            case pen_code of
              1 : begin
                    create_draw (old_x,old_y,x,y,d_code);
                    old_x := x;
                    old_y := y;
                  end;
              2 : begin                       { 'pen up' move }
                    old_x := x;
                    old_y := y;
                  end;
              3 : create_flash(x,y,d_code);
            end;              { case pen_code }
            end;              { if else begin }
          end;                { if achar = d  }
          if (achar = 'y') or (achar = 'Y') then begin
        {$ifdef verbose }
            writeln(output,' read_gerber, getting Y after X coord.');
        {$endif}
            l_y := getnum(filevar,achar);
        {$ifdef verbose }
            writeln(output,' read_gerber, Y coord = ',y,' term char =>',achar,'<');
        {$endif}
            case y_fmt_rt of
              2 : l_y := l_y * 10;
              3 : ;
              4 : l_y := l_y div 10;
              5 : l_y := l_y div 100;
              6 : l_y := l_y div 1000;
            end;                { case y_fmt_rt }
            y := l_y;
            if (achar = 'd') or (achar = 'D') then begin
              d_num := getnum(filevar,achar);
              if d_num < 4 then begin
        { this is pen up / pen down command on end of line }
              pen_code := d_num;
              {$ifdef verbose}
                writeln(output,'Read_Gerber pen code = ',pen_code);
              {$endif}
              case pen_code of
                1 : begin
                      create_draw (old_x,old_y,x,y,d_code);
                      old_x := x;
                      old_y := y;
                    end;
                2 : begin                       { 'pen up' move }
                      old_x := x;
                      old_y := y;
                    end;
                3 : create_flash(x,y,d_code);
              end;              { case pen_code }
            end else begin
        { select an aperture }
               if d_num > max_aperture then
                  writeln(output,'Specified aperture number ',d_num,' too high in line ',
                line_num,' of Gerber file.')
               else
                  d_code := d_num;
                  {$ifdef verbose}
                    writeln(output,'Read_Gerber aperture = ',d_num);
                  {$endif}
             end;               { if (achar = 'd') }
              end;              { if (achar = 'y') }
        end;                   {if  begin }
       end;             { 'x' }
    'y','Y' : begin
        y := getnum(filevar,achar);
        case y_fmt_rt of
          2 : y := y * 10;
          3 : ;
          4 : y := y div 10;
          5 : y := y div 100;
          6 : y := y div 1000;
        end;            { case y_fmt_rt }
        if (achar = 'd') or (achar = 'D') then begin
          d_num := getnum(filevar,achar);
          if d_num >= 4 then
            writeln(output,'unexpected pen change in coordinate line # ',
                line_num)
          else begin
          pen_code := d_num;
          {$ifdef verbose}
            writeln(output,'Read_Gerber pen code = ',pen_code);
          {$endif}
          case pen_code of
            1 : begin
                  create_draw (old_x,old_y,x,y,d_code);
                  old_x := x;
                  old_y := y;
                end;
            2 : begin                       { 'pen up' move }
                  old_x := x;
                  old_y := y;
                end;
            3 : create_flash(x,y,d_code);
          end;              { case pen_code }
        end;              { if else begin }
        end;                { if achar = d  }
      end;              { 'y' }
    end;                        { case achar }
    line_num := line_num + 1;
    readln(filevar);
  end;                  { while }
  read_gerber := line_num;
  writeln(output,' Gerber numeric format, X ',x_format,'    Y ',y_format);
end;                    { proc read_gerber }

{*section - following procedures for manipulating raster data }

Procedure bit_blt_or_inner (x,y : integer;
                this_dcode :dcode_ptr_type;
                var bits_untyp);

{ inserts 1 bits into zeroed band buffer }
type
  bits_type = array [0..maxint] of byte;
{ prepare a typecast for UNTYPED variable param bits_untyp }
var
  x_count,y_count : integer;
  ix_min,ix_max,iy_min,iy_max : integer;
  ix,iy,icol,irow,I,j,bit_offset,line_offset : integer;
  a,b,ps_xcent,ps_ycent : integer;
  flash_xmin,flash_xmax,flash_ymin,flash_ymax : integer;
  y_product,band_line : integer;
begin
{  writeln(output,'Bit_Blt_Or, x,y D# ',x:6,y:6,this_dcode^.d_num:4);}
  { calc extreme edges of plot space where aperture bitmap overlays }
  ps_xcent := x - x_offset;
  flash_xmin := ps_xcent - this_dcode^.x_center;
  flash_xmax := flash_xmin + this_dcode^.x_size - 1;
  ps_ycent := y - band_yoffset;
  flash_ymax := ps_ycent + this_dcode^.y_center; { note reversal of axes }
  if this_dcode^.aperture = thermal then
    flash_ymin := flash_ymax - this_dcode^.x_size +1 { plotter goes from high y to low }
  else
    flash_ymin := flash_ymax - this_dcode^.y_size +1; { plotter goes from high y to low }
  irow := 0;            { x index in aperture }
  if flash_xmin < 0 then begin
    irow := - flash_xmin;       { correct x index }
    flash_xmin := 0;
  end;
  ix_max :=  this_dcode^.x_size;
  if flash_xmax > line_length-1 then
    ix_max := ix_max - flash_xmax + line_length-1;
  icol := 0;
  if num_beams = 1 then begin
    if this_dcode^.aperture = thermal then
      iy_max :=  this_dcode^.x_size
    else
      iy_max :=  this_dcode^.y_size;
    if flash_ymin < 0 then begin
      icol := - flash_ymin;
      flash_ymin := 0;
    end;
    if flash_ymax > 0 then begin { otherwise, nothing to draw }
    if flash_ymax > band_width-1 then	{ ***** problem here ? *****}
      iy_max := iy_max - flash_ymax + band_width -1;
    band_line := flash_ymin;
    for iy := icol to iy_max -1 do begin	{ ***** prob here ? ****}
      y_product := iy * this_dcode^.x_size;
      line_offset := flash_xmin;
      for ix := irow to ix_max-1 do begin
{        writeln(output,'doing bitblt, ystart xstart ',band_line:6,flash_xmin:8);}
        if (band_line < 0) or (band_line > band_width ) or
            (line_offset < 0) or (line_offset > line_length - 1) then begin
            writeln(output,'range error in bit_blt_or, line, x addr',
                                  band_line:6,line_offset:6);
        end else begin
          if which_band = 1 then begin
            if pos_neg = positive then begin
                band1_array[band_line]^[line_offset] :=
                band1_array[band_line]^[line_offset]  or
                bits_type (bits_untyp) [y_product + ix];
            end else begin
                band1_array[band_line]^[line_offset] :=
                band1_array[band_line]^[line_offset]  and
                not (bits_type (bits_untyp) [y_product + ix]);
            end;
          end else begin
            if pos_neg = positive then begin
                  band2_array[band_line]^[line_offset] :=
                  band2_array[band_line]^[line_offset] or
                  bits_type(bits_untyp)[y_product + ix];
            end else begin
                  band2_array[band_line]^[line_offset] :=
                  band2_array[band_line]^[line_offset]  and
                  not (bits_type (bits_untyp) [y_product + ix]);
            end;                        { if pos_neg }
         end;                   { if which_band }
         line_offset := line_offset + 1;
        end;                    { if band_line < 0 }
      end;                              { for ix }
      band_line := band_line + 1;
    end;                                { for iy }
   end;                                 { if flash_ymax > 0 }
  end else begin                                { num beams = 8 }
        { ***** this section needs fixing ***** }
    iy_min := (this_dcode^.y_center - this_dcode^.y_radius)
           div 8 - y_offset;
    if iy_min < 0 then begin
      icol := - iy_min;
      iy_min := 0;
    end;
    iy_max := (this_dcode^.y_center + this_dcode^.y_radius)
           div 8 - y_offset;
    if iy_max > band_width then
        iy_max := band_width - 1;
    bit_offset := (this_dcode^.y_center + this_dcode^.y_radius) mod 8;
    for iy := iy_min to iy_max do begin
      i := irow;
      line_offset := x_count * i;
      for ix := ix_min to ix_max do begin
        if which_band = 1 then
          if bit_offset = 0 then
            band1_array[iy]^[ix] := band1_array[iy]^[ix] or
            bits_type (bits_untyp) [i+line_offset]
          else begin
        a := bits_type (bits_untyp) [i+line_offset] shl bit_offset;
        b := bits_type (bits_untyp) [i+line_offset] shr 7-bit_offset;
        a := a or b;
        band1_array[iy]^[ix]  := band1_array[iy]^[ix] or a;
          end
        else if bit_offset = 0 then
            band2_array[iy]^[ix] := band2_array[iy]^[ix] or
            bits_type (bits_untyp) [i+line_offset]
          else begin
        a := bits_type (bits_untyp) [i+line_offset] shl bit_offset;
        b := bits_type (bits_untyp) [i+line_offset] shr 7-bit_offset;
        a := a or b;
        band2_array[iy]^[ix]  := band2_array[iy]^[ix] or a;
        end;
        i := i + 1;
      end;                      { for ix := }
      icol := icol + 1;
    end;                                { for iy := }
  end;                          { if num_beams }
end;                            { proc bit_blt_or }

Procedure bit_blt_or (x,y : integer;
                this_dcode :dcode_ptr_type);
{ this is a dummy shell to pass the aperture array as untyped to the
        real routine }
begin
  bit_blt_or_inner(x,y,this_dcode,this_dcode^.bit_block^);
end;

function clipln(var x1,y1,x2,y2 : integer;awindow : window): boolean;
var
    start_mask,end_mask : set of (xlo,xhi,ylo,yhi);
    xi1,yi1,xi2,yi2 : integer;
    m,minv : real;
begin
    start_mask := [];           { clear mask}
    end_mask := [];
    {$ifdef clipln}
    writeln(output,'clipln x1,y1,x2,y2 ',x1:6,y1:6,x2:6,y2:6);
    {$endif}
    xi1 := x1;
    yi1 := y1;
    xi2 := x2;
    yi2 := y2;
    with awindow do begin
        if x1<xmin then start_mask := start_mask+[xlo];
        if x1>xmax then start_mask := start_mask+[xhi];
        if y1<ymin then start_mask := start_mask+[ylo];
        if y1>ymax then start_mask := start_mask+[yhi];
        if x2<xmin then end_mask := end_mask+[xlo];
        if x2>xmax then end_mask := end_mask+[xhi];
        if y2<ymin then end_mask := end_mask+[ylo];
        if y2>ymax then end_mask := end_mask+[yhi];
    end;                        { with awindow}
    if start_mask*end_mask <> [] then
        x1 := -1000          { out of window flag }
    else if (start_mask<>[]) or (end_mask<>[]) then begin
            if (x2-x1<>0) then m := (y2-y1)/(x2-x1)
                else m:=(y2-y1)/0.001;
            if m=0.0 then minv := 1000.0
                else minv := 1/m;
            if minv > 1000.0 then minv := 1000.0;
            with awindow do begin
                if xlo in start_mask then begin
                    yi1 := round((xmin-x1)*m+y1);
                    xi1 := xmin;
                end;
                if xhi in start_mask then begin
                    yi1 := round((xmax-x1)*m+y1);
                    xi1 := xmax;
                end;
                if ylo in start_mask then begin
                    xi1 := round((ymin-y1)*minv+x1);
                    yi1 := ymin;
                end;
                if yhi in start_mask then begin
                    xi1 := round((ymax-y1)*minv+x1);
                    yi1 := ymax;
                end;
                if xlo in   end_mask then begin
                    yi2 := round((xmin-x2)*m+y2);
                    xi2 := xmin;
                end;
                if xhi in   end_mask then begin
                    yi2 := round((xmax-x2)*m+y2);
                    xi2 := xmax;
                end;
                if ylo in   end_mask then begin
                    xi2 := round((ymin-y2)*minv+x2);
                    yi2 := ymin;
                end;
                if yhi in   end_mask then begin
                    xi2 := round((ymax-y2)*minv+x2);
                    yi2 := ymax;
                end;
            end;                {with awindow}
    end;
    with awindow do
    if  (((xi1<xmin) or (xi1>xmax)) or ((yi1<ymin) or (yi1>ymax)) or
        ((xi2<xmin) or (xi2>xmax)) or ((yi2<ymin) or (yi2>ymax))) then begin
                clipln := false;         { out of window flag}
      {$ifdef clipln}
      writeln(output,'clipln x1,y1,x2,y2 ',x1:6,y1:6,x2:6,y2:6,' false');
      {$endif}
    end else begin
      {$ifdef clipln}
      writeln(output,'clipln x1,y1,x2,y2 ',x1:6,y1:6,x2:6,y2:6,' true');
      {$endif}
        clipln := true;
        x1 := xi1;
        y1 := yi1;
        x2 := xi2;
        y2 := yi2;
    end;
end;  {proc clipln}

procedure window_band (var return_window : window);
{ Can you believe a TP 5.0 function can't return a record ?}
{ calculates window vertices of current marking band, so draw routines
        can clip to it }
begin
  return_window.xmin := 0;
  return_window.xmax := line_length-1;
  return_window.ymin := band_yoffset;
  return_window.ymax := band_yoffset + band_width;
end;

procedure draw_flash(a_flash : flash_ptr_type ; draw_window : window);
{ copies bitmap from appropriate aperture to the appropriate
  raster band buffer }
var
  i,j,k : integer;
  flash_xmin,flash_ymin,flash_xmax,flash_Ymax : integer;
  aperture_left_clip, aperture_right_clip : integer;
begin
  bit_blt_or(a_flash^.x,a_flash^.y,a_flash^.this_apt);
end;            { proc draw_flash }

function intercept(x1,y1,x2,y2,yint : integer ) : integer;
{ given line segment (x1,y1) to (x2,y2) and a y coordinate y_int,
    find the x axis value where line passes through y = y_int }
var
  del_x,del_y : integer;
begin
  del_x := x2 - x1;
  del_y := y1 - y2;
  if del_y = 0 then
    intercept := x1
  else
    intercept := x1 + round(((y1-yint) / del_y) * del_x);
end;

procedure rasfill_line(x_start,x_end, y : integer);
{ fills band line [y] from x_start to x_end }
var
  j : integer;
  bp : band_ptr_type;
begin
  if x_start < 0 then x_start := 0;
  if x_start > line_length -1 then x_start := line_length -1;
  if x_end > line_length -1 then x_end := line_length -1;
  if x_start > x_end then x_start := x_end-1;
  y := y - band_yoffset;
  if (y < lines_band) and (y >= 0) then begin
    if which_band = 1 then
       bp := band1_array[y]
    else
       bp := band2_array[y];
    for j := x_start to x_end do begin
      if pos_neg = positive then
          bp^[J] := 1
      else
          bp^[J] := 0;
    end;                        { for }
  end;                  { if (y < lines_band }
end;                    { proc rasfill_line }

procedure dual_bresenham(top_left_x,top_left_y,bot_left_x,bot_left_y,
                                         top_rt_x,top_rt_y,bot_rt_x,bot_rt_y : integer);
{ cranks out 2 lines in parallel, then fills the space between them }
{ will accept 'twisted' polygons, where top or bottom has top or bottom
	left x greater than same rt_x }
var
  left_d,left_dx,left_ai,left_bi,left_xi,left_x,y,dy,yi,j : integer;
  rt_d,rt_dx,rt_ai,rt_bi,rt_xi,rt_x : integer;
  left_lasty,rt_lasty,save_y : integer;
begin
  if (top_left_y <> top_rt_y) or (bot_left_y <> bot_rt_y) then
        writeln(output,' Y coord mismatch in dual bresenham.')
  else begin
    if (bot_left_x > bot_rt_x ) then begin
	j := bot_left_x;
	bot_left_x := bot_rt_x;
	bot_rt_x := j;
    end;
    if (top_left_x > top_rt_x ) then begin
	j := top_left_x;
	top_left_x := top_rt_x;
	top_rt_x := j;
    end;
   if (top_left_y = bot_left_y) then begin
      { only one line to process }
      rasfill_line(top_left_x,top_rt_x,top_left_y);
   end else begin
    {$ifdef bresen}
    writeln(output,'Bresenham ',top_left_x:5,top_left_y:5,
          bot_left_x:5,bot_left_y:5);
    writeln(output,'          ',top_rt_x:5,top_rt_y:5,bot_rt_x:5,
          bot_rt_y:5);
    {$endif}
    left_x := top_left_x;
    rt_x := top_rt_x;
    y := top_left_y;
    if (top_left_x < bot_left_x) then begin
      left_xi :=  1;
      left_dx := bot_left_x - top_left_x;
    end else begin
      left_xi :=  -1;
      left_dx := top_left_x - bot_left_x;
    end;
    if (top_rt_x < bot_rt_x) then begin
      rt_xi :=  1;
      rt_dx := bot_rt_x - top_rt_x;
    end else begin
      rt_xi :=  -1;
      rt_dx := top_rt_x - bot_rt_x;
    end;
    if top_left_y < bot_left_y then begin
      yi := 1;
      dy := bot_left_y - top_left_y;
    end else begin
      yi := -1;
      dy := top_left_y - bot_left_y;
    end;
    {$ifdef bresen}
    writeln(output,'bresenham first rasfill ',left_x:6,rt_x:6,y:6);
    {$endif}
    rasfill_line(left_x,rt_x,y);    { the initial dots }
    if left_dx > dy then begin
      left_ai := (dy - left_dx ) * 2;
      left_bi := dy * 2;
      left_d  := left_bi - left_dx;
    end else begin
      left_ai := (left_dx - dy ) * 2;
      left_bi := left_dx * 2;
      left_d  := left_bi - dy;
    end;
    if rt_dx > dy then begin
      rt_ai := (dy - rt_dx) * 2;
      rt_bi := dy * 2;
      rt_d  := rt_bi - rt_dx;
    end else begin
      rt_ai := (rt_dx - dy ) * 2;
      rt_bi := rt_dx * 2;
      rt_d  := rt_bi - dy;
    end;
    left_lasty := y;
    rt_lasty := y;
    repeat
      save_y := y;
      repeat
        if left_dx > dy then begin
          if (left_d >= 0) then begin
            inc(y,yi);
            inc(left_d,left_ai);
          end else begin
            inc(left_d,left_bi);
          end;
          inc(left_x,left_xi);
        end else begin
          if (left_d >= 0) then begin
            inc(left_x,left_xi);
            inc(left_d,left_ai);
          end else begin
            inc(left_d,left_bi);
          end;
          inc(y,yi);
        end;
      until y <> left_lasty;
      left_lasty := y;
      y := save_y;
      repeat
        if rt_dx > dy then begin
          if (rt_d >= 0) then begin
            inc(y,yi);
            inc(rt_d,rt_ai);
          end else begin
            inc(rt_d,rt_bi);
          end;
          inc(rt_x,rt_xi);
        end else begin
          if (rt_d >= 0) then begin
            inc(rt_x,rt_xi);
            inc(rt_d,rt_ai);
          end else begin
            inc(rt_d,rt_bi);
          end;
          inc(y,yi);
        end;
      until y <> rt_lasty;
      rt_lasty := y;
      {$ifdef bresen}
      writeln(output,'bresenham calls rasfill ',left_x:6,rt_x:6,y:6);
      {$endif}
      rasfill_line(left_x,rt_x,y);
    until (y = bot_left_y);
   end;
  end;
end;

procedure draw_line(a_line : line_ptr_type ; draw_window : window);
{ evaluates appropriate aperture, then draws clipped segment
            of line (in that width) to appropriate raster
            band buffer }
{ procedure draw_trace(atrace : trace_ptr; a_window : window);}
var
    x1,x2,y1,y2 : integer;
    antiangle : real;
    wstep,side,nx1,ny1,nx2,ny2 : integer;
    poly_x,poly_y : poly_array;
    ix,iy,ix1,iy1,ix2,iy2,a     : integer;
    cangle,sangle : real;
    draw_xmin,draw_xmax,draw_ymin,draw_ymax : integer;
    trit_top_y,trit_bot_y,trit_lt_x,trit_rt_x : integer;
    trib_top_y,trib_bot_y,trib_lt_x,trib_rt_x : integer;
    sort_flag : boolean;
    i : integer;
    bp : band_ptr_type;
begin
    with draw_window do begin
        x1:=a_line^.x1 - x_offset; y1:=a_line^.y1;
        x2:=a_line^.x2 - x_offset; y2:=a_line^.y2;
    end;        {with}
 {   if clipln(x1,y1,x2,y2,draw_window) then begin}
    {$ifdef line_dump}
    writeln(output,'draw_line ',x1:6,y1:6,'  ',x2:6,y2:6);
    {$endif}
{ draw flashes at the endpoints to round end of trace }
        bit_blt_or(a_line^.x1,y1,a_line^.this_apt);
        bit_blt_or(a_line^.x2,y2,a_line^.this_apt);
    {if line is entirely outside window, assume 2d trace is too}
    {find 1/slope}
        if y1=y2 then antiangle:=arctan(10000.0) {prevent div by 0 fault}
        else  antiangle:=-arctan((x1-x2)/(y1-y2));
        {$ifdef line_dump}
        writeln(output,'antiangle ',antiangle);
        {$endif}
        cangle := cos(antiangle);
        sangle := sin(antiangle);
        wstep:=round(a_line^.width/2.0);
        side:=-wstep;
        poly_x[1]:=round(x1+side*cangle);
        poly_y[1]:=round(y1+side*sangle);
        poly_x[4]:=round(x2+side*cangle);
        poly_y[4]:=round(y2+side*sangle);
        side:=wstep;
        poly_x[2]:=round(x1+side*cangle);
        poly_y[2]:=round(y1+side*sangle);
        poly_x[3]:=round(x2+side*cangle);
        poly_y[3]:=round(y2+side*sangle);
 { sort 4 poly vertices in order of descending Y coords }
        repeat
          sort_flag := false;
          for i := 1 to 3 do
            if poly_y[i] < poly_y[i+1] then begin
              a := poly_y[i+1];
              poly_y[i+1] := poly_y[i];
              poly_y[i] := a;
              a := poly_x[i+1];
              poly_x[i+1] := poly_x[i];
              poly_x[i] := a;
              sort_flag := true;
            end;                        { if }
        until not sort_flag;
        draw_ymax := poly_y[1];
        draw_ymin := poly_y[4];
        draw_xmin :=  25000;
        draw_xmax := -25000;
        for i := 1 to 4 do begin
          if poly_x[i] > draw_xmax then draw_xmax := poly_x[i];
          if poly_x[i] < draw_xmin then draw_xmin := poly_x[i];
        end;
{ now fill area enclosed by poly_x, poly_y arrays }
  { convert plot space edges to band space edges }
        draw_ymin := draw_ymin - band_yoffset;
        draw_ymax := draw_ymax - band_yoffset;
        {$ifdef line_dump }
        writeln(output,'draw shifted',draw_xmin:6,draw_ymin:6,'   ',
            draw_xmax:6,draw_ymax:6);
        writeln(output,'window_ymin,ymax ',draw_window.ymin:6,
            draw_window.ymax:6);
        writeln(output,'check xy ',x1:6,y1:6,'  ',x2:6,y2:6);
        {$endif}
  { window first to see if any part could be in band }
        if (draw_ymin <= band_width-1) and (draw_ymax  >= 0) then begin
          if draw_ymin < 0 then draw_ymin := 0;
          if draw_ymax > band_width -1 then draw_ymax := band_width-1;
          if (x1 = x2) or (y1 = y2) then begin  { aligned horiz or vert line }
            {$ifdef line_dump }
            writeln(output,'drawing line, x1,y1,x2,y2 ',
                draw_xmin:6,draw_ymin:6,'    ',draw_xmax:6,draw_ymax:6);
            {$endif}
            if draw_xmin < 0 then draw_xmin := 0;  { protect array bounds }
            if draw_xmax > line_length-1 then draw_xmax := line_length-1;
            for i := draw_ymin to draw_ymax do begin
              if which_band = 1 then
                bp := band1_array[i]
              else
                bp := band2_array[i];
              if pos_neg = positive then
                for j := draw_xmin to draw_xmax do
                    bp^[J] := bp^[J] or 1
              else
                for j := draw_xmin to draw_xmax do
                    bp^[J] := bp^[J] and not (1);
            end;			{ for i := }
        end else begin  { a line with 0 < slope < infinity}
{***** now have to fill area as 3 pieces, a triangle at top,a parallelogram,
                and a triangle at bottom }
          {Poly[1] is at top, but we have to find whether poly[2] or poly[3] is left }
          if poly_x[2] < poly_x[1] then begin           { poly[2] is left side }
  { left line segment is complete, but we have to find intercept on right line seg
          {             that has same Y coord as poly_y[2]      }
            trit_top_y := poly_y[1];
            trit_bot_y := poly_y[2];
            trit_lt_x := poly_x[2];
            trit_rt_x := intercept(poly_x[1],poly_y[1],poly_x[3],poly_y[3],poly_y[2]);
            dual_bresenham(poly_x[1],poly_y[1],trit_lt_x,trit_bot_y,
                        poly_x[1],poly_y[1],trit_rt_x,trit_bot_y);
          { now we need to find intercept on left seg across from poly_y[3] }
            trib_top_y := poly_y[3];
            trib_bot_y := poly_y[4];
            trib_rt_x := poly_x[3];
            trib_lt_x := intercept(poly_x[2],poly_y[2],poly_x[4],poly_y[4],poly_y[3]);
            dual_bresenham(trib_lt_x,trib_top_y,poly_x[4],poly_y[4],
                        trib_rt_x,trib_top_y,poly_x[4],poly_y[4]);
          { finally, fill in center parallelogram part }            
            dual_bresenham(poly_x[2],poly_y[2],trib_lt_x,trib_top_y,
                                     trit_rt_x,trit_bot_y,poly_x[3],poly_y[3]);
          end else begin
   { right line segment is complete, but we have to find intercept on left line seg
          {             that has same Y coord as poly_y[2]      }
            trit_top_y := poly_y[1];
            trit_bot_y := poly_y[2];
            trit_rt_x := poly_x[2];
            trit_lt_x := intercept(poly_x[1],poly_y[1],poly_x[3],poly_y[3],poly_y[2]);
            dual_bresenham(poly_x[1],poly_y[1],trit_lt_x,trit_bot_y,
                        poly_x[1],poly_y[1],trit_rt_x,trit_bot_y);
          { now we need to find intercept on right seg across from poly_y[3] }
            trib_top_y := poly_y[3];
            trib_bot_y := poly_y[4];
            trib_lt_x := poly_x[3];
            trib_rt_x := intercept(poly_x[2],poly_y[2],poly_x[4],poly_y[4],poly_y[3]);
            dual_bresenham(trib_lt_x,trib_top_y,poly_x[4],poly_y[4],
                        trib_rt_x,trib_top_y,poly_x[4],poly_y[4]);
          { finally, fill in center parallelogram part }            
            dual_bresenham(trit_lt_x,trit_bot_y,poly_x[3],poly_y[3],
                                     poly_x[2],poly_y[2],trib_rt_x,trib_top_y);
          end;          { if poly_x[2] < }
        end;            { if (x1 = x2... }
      end;              { if draw_ymin }
 {   end;                {if clipln...}
end;                    {proc draw_line}

procedure make_plot;
var
  WordVal : integer;
begin
  {$ifdef dump_data}
      assign(picture,'picture.dat');
      rewrite(picture);
        write(picture,line_length,'    ');
	WordVal := main_window.ymax-main_window.ymin;
	if (WordVal < 0) then WordVal := -WordVal;
	WordVal := WordVal + 2* border;
        writeln(picture,WordVal);
  {$endif}
  which_band := 1;
  band_yoffset := y_offset;
  done := false;
  repeat
    writeln(output,'marking band # ',which_band,'   line # ',
                band_yoffset);
    clear_band(which_band);         { clear it to background }
{$ifdef dump_band_lots}
    for i := 0 to lines_band-1 do begin
      writeln(output,'band1_array[',i,'] => ',seg(band1_array[i]^),
             ':',ofs(band1_array[i]^));
      writeln(output,'band2_array[',i,'] => ',seg(band2_array[i]^),
             ':',ofs(band2_array[i]^));
    end;
{$endif}
    window_band(curr_window);
    flash_1 := flash_head;          { draw all flashes }
    if flash_1 <> nil then begin
      repeat
          draw_flash(flash_1,curr_window);
          flash_1 := flash_1^.next_flash;
      until (flash_1 = nil);
    {$ifdef debug}
        writeln(output,'all flashes done, now doing draws.');
    {$endif}
    end;
    draw_1 := line_head;            { draw all draws }
    if draw_1 <> nil then begin
      repeat
          draw_line(draw_1,curr_window);
          draw_1 := draw_1^.next_line;
      until (draw_1 = nil);
    end;
  {$ifndef dump_data}
    check_band(which_band); { wait for this band to be done plotting }
    send_band(which_band);
  {$endif}
    {$ifdef dump_data}
    i := band_width -1;  { since we are going down in Y, must go down }
    while i >= 0 do begin     { in band, too }
      j:= 0;
      while (j < line_length-4) and (j < max_line) do begin
        bflag := false;
             k := 0;
         {   for k := 0 to 1 do begin   }
              if which_band = 1 then begin
                if (band1_array[i]^[j+k] and 1 ) = 1 then bflag := true;
              end else begin
                if (band2_array[i]^[j+k] and 1) = 1 then bflag := true;
              end;
        {    end;             { for k }
            if bflag then write(picture,'*')
                else write(picture,' ');
            j := j + 1;
          end;  { while j < }
          writeln(picture);
          i := i - 1;
        end;    { for i }
    {$endif}
    last_band := which_band;
    if which_band = 1 then
      which_band := 2
    else
      which_band := 1;
      if band_yoffset < main_window.ymin - border then
          done := true;
        band_yoffset := band_yoffset - band_width;
  until done;
  if which_band = 1 then
    which_band := 2
  else
    which_band := 1;
  check_band(last_band);
  writeln(output,' plot is done.');
  return_mem;
end;		{ proc make_plot }

end.
